@@ -22,6 +22,7 @@
 
 import __main__
 
+from functools import partial
 import sys
 import os
 from urllib.parse import quote as url_quote
@@ -34,10 +35,16 @@ import time
 import socket
 import shutil
 import shlex
-from typing import (Optional, Union, Tuple)
+from typing import (List, Optional, Union, Tuple)
 
 import adtlog
 
+# When running installed, this is /usr/share/autopkgtest.
+# When running uninstalled, this is the source tree.
+# Either way, it has a lib subdirectory from which virtualization
+# backends can read extra files.
+PKGDATADIR = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
+
 progname = "<VirtSubproc>"
 devnull_read = open('/dev/null', 'rb')
 caller = __main__
@@ -58,15 +65,17 @@ class Quit(RuntimeError):
 
 
 class Timeout(RuntimeError):
-    pass
+
+    def __init__(self, timeout_secs):
+        self.timeout_secs = timeout_secs
 
 
-def alarm_handler(*a):
-    raise Timeout()
+def alarm_handler(to, *a):
+    raise Timeout(to)
 
 
 def timeout_start(to):
-    signal.signal(signal.SIGALRM, alarm_handler)
+    signal.signal(signal.SIGALRM, partial(alarm_handler, to))
     signal.alarm(to)
 
 
@@ -195,6 +204,58 @@ def check_exec(argv, downp=False, outp=False, timeout=0, fail_on_stderr=True):
     return out
 
 
+def load_shell_script(path: str) -> str:
+    if not os.path.isabs(path):
+        path = os.path.join(PKGDATADIR, path)
+
+    with open(path) as reader:
+        return reader.read()
+
+
+def wait_booted(
+    exec_command: List[str],
+    timeout: int = 120,
+    short_command_timeout: int = 10,
+    long_command_timeout: int = 60,
+) -> None:
+    '''
+    Wait until a container or VM with an init system has sufficiently
+    booted to interact with it.
+
+    timeout: how long we will continue to poll the container for status
+
+    short_command_timeout: how long we will wait for a relatively quick command
+    to finish
+
+    long_command_timeout: how long we will wait for a slower command to finish
+    '''
+    await_boot = load_shell_script('lib/in-testbed/await-boot.sh')
+
+    while timeout > 0:
+        timeout -= 1
+        time.sleep(1)
+        (rc, out, _) = execute_timeout(
+            None, 10, exec_command + ['runlevel'],
+            stdout=subprocess.PIPE,
+        )
+        if rc != 0:
+            adtlog.debug('wait_booted: polling status failed, retrying...')
+            continue
+        out = out.strip()
+        if out.split()[-1].isdigit():
+            adtlog.debug('waiting for network')
+            check_exec(
+                exec_command + ['sh', '-ec', await_boot],
+                timeout=long_command_timeout,
+            )
+            return
+
+        adtlog.debug('wait_booted: runlevel "%s", retrying...' % out)
+
+    bomb('timed out waiting for testbed to start; '
+         'last runlevel "%s"' % out)
+
+
 class timeout:
     def __init__(self, secs, exit_msg=None):
         '''Context manager that times out after given number of seconds.
@@ -703,8 +764,8 @@ def command():
     print(' '.join(r))
 
 
-signal_list = [	signal.SIGHUP, signal.SIGTERM,
-                signal.SIGINT, signal.SIGPIPE]
+signal_list = [signal.SIGHUP, signal.SIGTERM,
+               signal.SIGINT, signal.SIGPIPE]
 
 
 def sethandlers(f):
