@@ -391,15 +391,23 @@ class Testbed:
             ''' % (pocket, pocket, pocket)
             self.check_exec(['sh', '-ec', script])
 
-        # create apt pinning for --apt-pocket with package list
+        # create apt pinning for --apt-pocket
         for pocket in self.add_apt_pockets:
             # do we have a package list?
             try:
                 (pocket, pkglist) = pocket.split('=', 1)
             except ValueError:
-                continue
-            self._create_apt_pinning_for_packages(self._get_default_release() + '-' + pocket, pkglist,
-                                                  [self._get_default_release() + '-updates'])
+                pkglist = ''
+
+            # set priority for the specified pocket.
+            # this is useful to override NotAutomatic: yes.
+            self._create_apt_pinning_for_pocket(self._get_default_release() + '-' + pocket)
+
+            # set priorities for individual packages if specified
+            if pkglist:
+                self._create_apt_pinning_for_packages(self._get_default_release() + '-' + pocket, pkglist,
+                                                      [self._get_default_release() + '-updates'])
+
         if self.add_apt_releases:
             get_mirror_and_components = '''
             sed -rn 's/^(deb|deb-src) +(\\[.*\\] *)?((http|https|file):[^ ]*) +([^ ]+) +(.*)$/\\2\\3 \\6/p' \\
@@ -1266,6 +1274,15 @@ Description: satisfy autopkgtest test dependencies
     # helper methods
     #
 
+    def _create_apt_pinning_for_pocket(self, release):
+        '''Create apt pinning for --apt-pocket=pocket[=pkglist] with
+           Pin-Priority: 500 to undo the effect of NotAutomatic: yes'''
+
+        script = 'mkdir -p /etc/apt/preferences.d; '
+        script += 'printf "Package: *\\nPin: release a=%(release)s\\nPin-Priority: 500\\n" > /etc/apt/preferences.d/autopkgtest-%(release)s-baseline; ' % \
+            {'release': release}
+        self.check_exec(['sh', '-ec', script])
+
     def _create_apt_pinning_for_packages(self, release, pkglist, default_releases=[]):
         '''Create apt pinning for --apt-pocket=pocket=pkglist and
            --pin-packages=release=pkglist'''
