@@ -156,13 +156,14 @@ class QemuImage:
 
         if self.overlay is None:
             bits.append('file={}'.format(self.file))
+            bits.append('format={}'.format(self.format))
         else:
             bits.append('file={}'.format(self.overlay))
+            bits.append('format=qcow2')
             bits.append('cache=unsafe')
 
         bits.append('if=virtio')
         bits.append('discard=unmap')
-        bits.append('format={}'.format(self.format))
 
         if self.readonly:
             bits.append('readonly')
@@ -368,12 +369,15 @@ class Qemu:
                     'unix:%s,server=on,wait=off' % self.get_socket_path('hvc0'),
                 ])
             else:
+                if hvc == 'hvc0' or self.qemu_architecture == 'ppc64le':
+                    argv.extend([
+                        '-device', 'virtio-serial',
+                    ])
                 argv.extend([
                     '-chardev',
                     'socket,path=%s,server=on,wait=off,id=%s' % (
                         self.get_socket_path(hvc), hvc,
                     ),
-                    '-device', 'virtio-serial',
                     '-device', 'virtconsole,chardev=%s' % hvc,
                 ])
 
