@@ -24,11 +24,9 @@ import string
 import re
 import errno
 import os.path
-import json
 import subprocess
 import tempfile
-import atexit
-import shutil
+from typing import (Dict, FrozenSet, Iterable, List, Optional, Union)
 
 import debian.deb822
 import debian.debian_support
@@ -40,11 +38,47 @@ import adtlog
 # Abstract test representation
 #
 
-known_restrictions = ['rw-build-tree', 'breaks-testbed', 'needs-root',
-                      'build-needed', 'allow-stderr', 'isolation-container',
-                      'isolation-machine', 'needs-recommends', 'needs-reboot',
-                      'flaky', 'skippable', 'superficial',
-                      'skip-not-installable', 'needs-internet']
+known_restrictions = frozenset({
+    'allow-stderr',
+    'breaks-testbed',
+    'build-needed',
+    'flaky',
+    'isolation-container',
+    'isolation-machine',
+    'needs-internet',
+    'needs-reboot',
+    'needs-recommends',
+    'needs-root',
+    'needs-sudo',
+    'rw-build-tree',
+    'skip-not-installable',
+    'skippable',
+    'superficial',
+})
+
+
+# Keys are restrictions
+# Values are sets of either:
+# - str: a capability that is required by the restriction
+# - set of two or more str: at least one of these caps is required
+#   by the restriction
+RESTRICTIONS_REQUIRE_CAPS = {
+    'breaks-testbed': frozenset({'revert-full-system'}),
+    'needs-internet': frozenset({'has_internet'}),
+    'needs-reboot': frozenset({'reboot'}),
+    'needs-root': frozenset({'root-on-testbed'}),
+    'isolation-container': frozenset({
+        frozenset({'isolation-container', 'isolation-machine'}),
+    }),
+    'isolation-machine': frozenset({'isolation-machine'}),
+}       # type: Dict[str, FrozenSet[Union[str, FrozenSet[str]]]]
+
+
+# Keys are restrictions
+# Values are sets of dependency strings
+RESTRICTIONS_IMPLY_DEPS = {
+    'needs-sudo': frozenset({'sudo'}),
+}
 
 
 class Unsupported(Exception):
@@ -81,8 +115,16 @@ class Test:
     This is only a representation of the metadata, it does not have any
     actions.
     '''
-    def __init__(self, name, path, command, restrictions, features, depends,
-                 clicks, installed_clicks, synth_depends):
+    def __init__(
+        self,
+        name: str,
+        path: Optional[str],
+        command: Optional[str],
+        restrictions: Iterable[str],
+        features: Iterable[str],
+        depends: List[str],
+        synth_depends: List[str],
+    ) -> None:
         '''Create new test description
 
         A test must have either "path" or "command", the respective other value
@@ -91,10 +133,8 @@ class Test:
         @name: Test name
         @path: path to the test's executable, relative to source tree
         @command: shell command for the test code
-        @restrictions, @features: string lists, as in README.package-tests
+        @restrictions, @features: sets of strings, as in README.package-tests
         @depends: string list of test dependencies (packages)
-        @clicks: path list of click packages to install for this test
-        @installed_clicks: names of already installed clicks for this test
         @synth_depends: string list of synthesized test dependencies (packages)
         '''
         if '/' in name:
@@ -106,20 +146,23 @@ class Test:
         self.name = name
         self.path = path
         self.command = command
-        self.restrictions = restrictions
-        self.features = features
-        self.depends = depends
-        self.clicks = clicks
-        self.installed_clicks = installed_clicks
+        self.restrictions = set(restrictions)
+        self.features = set(features)
+        self.depends = list(depends)
         self.synth_depends = synth_depends
+
+        for r in self.restrictions:
+            for d in RESTRICTIONS_IMPLY_DEPS.get(r, frozenset()):
+                if d not in self.depends:
+                    self.depends.append(d)
+
         # None while test hasn't run yet; True: pass, False: fail
         self.result = None
         self.skipped = False
         adtlog.debug('Test defined: name %s path %s command "%s" '
-                     'restrictions %s features %s depends %s clicks %s '
-                     'installed clicks %s' %
-                     (name, path, command, restrictions, features, depends,
-                      clicks, installed_clicks))
+                     'restrictions %s features %s depends %s ' %
+                     (name, path, command, sorted(restrictions),
+                      sorted(features), depends))
 
     def passed(self):
         '''Mark test as passed'''
@@ -154,47 +197,39 @@ class Test:
         Raise Unsupported exception if there are any.
         '''
         effective = set(self.restrictions) - set(ignore_restrictions)
+        provided = set(caps)
 
         for r in effective:
             if r not in known_restrictions:
                 raise Unsupported(self.name, 'unknown restriction %s' % r)
 
-        if 'isolation-container' in effective and \
-           'isolation-container' not in caps and \
-           'isolation-machine' not in caps:
-            raise Unsupported(self.name,
-                              'Test requires container-level isolation but '
-                              'testbed does not provide that')
+        for r in effective:
+            need_caps = RESTRICTIONS_REQUIRE_CAPS.get(r, frozenset())
 
-        if 'isolation-machine' in effective and \
-           'isolation-machine' not in caps:
-            raise Unsupported(self.name,
-                              'Test requires machine-level isolation but '
-                              'testbed does not provide that')
+            for c in need_caps:
+                if isinstance(c, str):
+                    if c not in caps:
+                        raise Unsupported(
+                            self.name,
+                            ('Test restriction "%s" requires testbed '
+                             'capability "%s"') % (r, c),
+                        )
+                else:
+                    # must have at least one of the capabilities
+                    assert isinstance(c, frozenset)
+                    assert len(c) > 1
 
-        if 'breaks-testbed' in effective and \
-           'revert-full-system' not in caps:
-            raise Unsupported(self.name,
-                              'Test breaks testbed but testbed does not '
-                              'provide revert-full-system')
+                    if not (c & provided):
+                        cap_names = sorted('"%s"' % n for n in c)
+                        needed = ', '.join(cap_names[:-1])
+                        needed += ' and/or ' + cap_names[-1]
 
-        if 'needs-root' in effective and \
-           'root-on-testbed' not in caps:
-            raise Unsupported(self.name,
-                              'Test needs root on testbed which is not '
-                              'available')
+                        raise Unsupported(
+                            self.name,
+                            ('Test restriction "%s" requires testbed '
+                             'capability %s') % (r, needed),
+                        )
 
-        if 'needs-reboot' in effective and \
-           'reboot' not in caps:
-            raise Unsupported(self.name,
-                              'Test needs to reboot testbed but testbed does '
-                              'not provide reboot capability')
-
-        if 'needs-internet' in self.restrictions and \
-           'has_internet' not in caps:
-            raise Unsupported(self.name,
-                              'Test needs unrestricted internet access but testbed does '
-                              'not provide it')
 
 #
 # Parsing for Debian source packages
@@ -270,17 +305,32 @@ def _debian_packages_from_source(srcdir):
     return (packages, packages_no_arch)
 
 
+def _filter_variables(deps):
+    deplist = []
+    for dep in [d.strip() for d in deps.split(',')]:
+        if re.match(r'^\${', dep):
+            continue
+        dep = re.sub(r'\(.*\${\S*}.*\)', '', dep)
+        deplist.append(dep)
+    return ', '.join(deplist)
+
+
 def _debian_recommends_from_source(srcdir, testbed_arch):
     deps = ''
     for st in parse_rfc822(os.path.join(srcdir, 'debian/control')):
         if 'Recommends' in st:
             deps += st['Recommends'] + ','
 
+    # Because we parse the source d/control file instead of the binary
+    # control file, we may run into variables. See bug 1008206 for
+    # some discussion.
+    deps = _filter_variables(deps)
+
     # resolve arch specific dependencies and build profiles
     perl = subprocess.Popen(['perl', '-'], stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE)
     code = '''use Dpkg::Deps;
-              $dep = deps_parse('%s', reduce_arch => 1,
+              $dep = deps_parse('%s', reduce_restrictions => 1,
                                 host_arch => '%s');
               $out = $dep->output();
               print $out, "\\n";
@@ -292,7 +342,7 @@ def _debian_recommends_from_source(srcdir, testbed_arch):
     deps = [d.strip() for d in deps.split(',')]
 
     # The following check is a bit silly as there were apparently no
-    # recommeds, but still the test asked for it. Let's not fail on
+    # recommends, but still the test asked for it. Let's not fail on
     # that, but also let's not add the empty string.
     if deps == ['']:
         deps = []
@@ -532,7 +582,8 @@ def _check_architecture(name, testbed_arch, architectures):
 
 
 def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
-                        auto_control=True, ignore_restrictions=(), testname=None):
+                        auto_control=True, ignore_restrictions=(),
+                        only_tests=()):
     '''Parse test descriptions from a Debian DEP-8 source dir
 
     @ignore_restrictions: If we would skip the test due to these restrictions,
@@ -581,8 +632,9 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
     for record in parse_rfc822(control_path):
         command = None
         try:
-            restrictions = record.get('Restrictions', '').replace(
-                ',', ' ').split()
+            restrictions = set(
+                record.get('Restrictions', '').replace(',', ' ').split()
+            )
 
             # needs-recommends is deprecated, but until removed, let's
             # replace it with @recommends@
@@ -590,13 +642,13 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
                 record['Depends'] = record.get('Depends', '@') + ',@recommends@'
 
             feature_test_name = None
-            features = []
+            features = set()
             record_features = record.get('Features', '').replace(
                 ',', ' ').split()
             for feature in record_features:
                 details = feature.split('=', 1)
                 if details[0] != 'test-name':
-                    features.append(feature)
+                    features.add(feature)
                     continue
                 if len(details) != 2:
                     # No value, i.e. a bare 'test-name'
@@ -606,7 +658,7 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
                     raise InvalidControl(
                         '*', 'only one test-name feature allowed')
                 feature_test_name = details[1]
-                features.append(feature)
+                features.add(feature)
             architectures = record.get('Architecture', '').replace(
                 ',', ' ').split()
 
@@ -633,10 +685,10 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
                         _check_architecture(n, testbed_arch, architectures)
 
                         test = Test(n, os.path.join(test_dir, n), None,
-                                    restrictions, features, depends, [], [], synth_depends)
+                                    restrictions, features, depends, synth_depends)
                         test.check_testbed_compat(testbed_caps, ignore_restrictions)
                     except Unsupported as u:
-                        if testname is None or n == testname:
+                        if (not only_tests) or n in only_tests:
                             u.report()
                             some_skipped = True
                     else:
@@ -656,153 +708,15 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
                 _debian_check_unknown_fields(name, record)
                 _check_architecture(name, testbed_arch, architectures)
                 test = Test(name, None, command, restrictions, features,
-                            depends, [], [], synth_depends)
+                            depends, synth_depends)
                 test.check_testbed_compat(testbed_caps, ignore_restrictions)
                 tests.append(test)
             else:
                 raise InvalidControl('*', 'missing "Tests" or "Test-Command"'
                                      ' field')
         except Unsupported as u:
-            if testname is None or n == testname:
+            if not only_tests:
                 u.report()
                 some_skipped = True
 
     return (tests, some_skipped)
-
-
-#
-# Parsing for click packages
-#
-
-def parse_click_manifest(manifest, testbed_caps, clickdeps, use_installed,
-                         srcdir=None, ignore_restrictions=(), testname=None):
-    '''Parse test descriptions from a click manifest.
-
-    @manifest: String with the click manifest
-    @testbed_caps: List of testbed capabilities
-    @clickdeps: paths of click packages that these tests need
-    @use_installed: True if test expects the described click to be installed
-                    already
-    @ignore_restrictions: If we would skip the test due to these restrictions,
-                          run it anyway
-    @testname: If we're asked a specific test, don't report on others
-
-    Return (source_dir, list of Test objects, some_skipped). If this encounters
-    any invalid restrictions, fields, or test restrictions which cannot be met
-    by the given testbed capabilities, the test will be skipped (and reported
-    so), and not be included in the result.
-
-    If srcdir is given, use that as source for the click package, and return
-    that as first return value. Otherwise, locate and download the source from
-    the click's manifest into a temporary directory and use that.
-
-    This may raise an InvalidControl exception.
-    '''
-    try:
-        manifest_j = json.loads(manifest)
-        test_j = manifest_j.get('x-test', {})
-    except ValueError as e:
-        raise InvalidControl(
-            '*', 'click manifest is not valid JSON: %s' % str(e))
-    if not isinstance(test_j, dict):
-        raise InvalidControl(
-            '*', 'click manifest x-test key must be a dictionary')
-
-    installed_clicks = []
-    if use_installed:
-        installed_clicks.append(manifest_j.get('name'))
-
-    some_skipped = False
-    tests = []
-
-    # It's a dictionary and thus does not have a predictable ordering; sort it
-    # to get a predictable list
-    for name in sorted(test_j):
-        desc = test_j[name]
-        adtlog.debug('parsing click manifest test %s: %s' % (name, desc))
-
-        # simple string is the same as { "path": <desc> } without any
-        # restrictions, or the special "autopilot" case
-        if isinstance(desc, str):
-            if name == 'autopilot' and re.match('^[a-z_][a-z0-9_]+$', desc):
-                desc = {'autopilot_module': desc}
-            else:
-                desc = {'path': desc}
-
-        if not isinstance(desc, dict):
-            raise InvalidControl(name, 'click manifest x-test dictionary '
-                                 'entries must be strings or dicts')
-
-        # autopilot special case: dict with extra depends
-        if 'autopilot_module' in desc:
-            desc['command'] = \
-                'PYTHONPATH=app/tests/autopilot:tests/autopilot:$PYTHONPATH '\
-                'python3 -m autopilot.run run -v -f subunit -o ' \
-                '$AUTOPKGTEST_ARTIFACTS/%s.subunit ' % name + os.environ.get(
-                    'AUTOPKGTEST_AUTOPILOT_MODULE',
-                    os.environ.get('ADT_AUTOPILOT_MODULE', desc['autopilot_module']))
-            desc.setdefault('depends', []).insert(
-                0, 'ubuntu-ui-toolkit-autopilot')
-            desc['depends'].insert(0, 'autopilot-touch')
-            if 'allow-stderr' not in desc.setdefault('restrictions', []):
-                desc['restrictions'].append('allow-stderr')
-
-        try:
-            test = Test(name, desc.get('path'), desc.get('command'),
-                        desc.get('restrictions', []), desc.get('features', []),
-                        desc.get('depends', []), clickdeps, installed_clicks, [])
-            test.check_testbed_compat(testbed_caps, ignore_restrictions)
-            tests.append(test)
-        except Unsupported as u:
-            if testname is None or name == testname:
-                u.report()
-                some_skipped = True
-
-    if srcdir is None:
-        # do we have an x-source/vcs-bzr link?
-        if 'x-source' in manifest_j:
-            try:
-                repo = manifest_j['x-source']['vcs-bzr']
-                adtlog.info('checking out click source from %s' % repo)
-                d = tempfile.mkdtemp(prefix='autopkgtest.clicksrc.')
-                atexit.register(shutil.rmtree, d, ignore_errors=True)
-                try:
-                    subprocess.check_call(['bzr', 'checkout', '--lightweight',
-                                           repo, d])
-                    srcdir = d
-                except subprocess.CalledProcessError as e:
-                    adtlog.error('Failed to check out click source from %s: %s'
-                                 % (repo, str(e)))
-            except KeyError:
-                adtlog.error('Click source download from x-source only '
-                             'supports "vcs-bzr" repositories')
-        else:
-            adtlog.error('cannot download click source: manifest does not '
-                         'have "x-source"')
-
-    return (srcdir, tests, some_skipped)
-
-
-def parse_click(clickpath, testbed_caps, srcdir=None, testname=None):
-    '''Parse test descriptions from a click package.
-
-    Return (source_dir, list of Test objects, some_skipped). If this encounters
-    any invalid restrictions, fields, or test restrictions which cannot be met
-    by the given testbed capabilities, the test will be skipped (and reported
-    so), and not be included in the result.
-
-    If srcdir is given, use that as source for the click package, and return
-    that as first return value. Otherwise, locate and download the source from
-    the click's manifest into a temporary directory and use that (not yet
-    implemented).
-
-    This may raise an InvalidControl exception.
-    '''
-    pkg = debian.debfile.DebFile(clickpath)
-    try:
-        manifest = pkg.control.get_content('manifest').decode('UTF-8')
-    finally:
-        pkg.close()
-
-    return parse_click_manifest(manifest, testbed_caps, [clickpath], False,
-                                srcdir, testname=testname)
